#ifndef _JZ_P32MX_CLKC_H_
#define _JZ_P32MX_CLKC_H_
#include <stdint.h>

#if defined(_USE_SFRS_BASE_ADDR)
  extern const void * _sfrs_base_addr_0;
  #define _clkc_base_addr        (_sfrs_base_addr_0 + 0xF000)
#else
  #define _clkc_base_addr	((const void *)0xBF80F000)
#endif

#define _CLKC_ADDR	(_clkc_base_addr)

  #define OSCCON           (*(volatile unsigned *)(_CLKC_ADDR + 0x00))
  #define OSCCON_CLR       (*(volatile unsigned *)(_CLKC_ADDR + 0x04))
  #define OSCCON_SET       (*(volatile unsigned *)(_CLKC_ADDR + 0x08))
  #define OSCCON_INV       (*(volatile unsigned *)(_CLKC_ADDR + 0x0C))
  #define OSCTUN           (*(volatile unsigned *)(_CLKC_ADDR + 0x10))
  #define OSCTUN_CLR       (*(volatile unsigned *)(_CLKC_ADDR + 0x14))
  #define OSCTUN_SET       (*(volatile unsigned *)(_CLKC_ADDR + 0x18))
  #define OSCTUN_INV       (*(volatile unsigned *)(_CLKC_ADDR + 0x1C))
  #define REFOCON          (*(volatile unsigned *)(_CLKC_ADDR + 0x20))
  #define REFOCON_CLR      (*(volatile unsigned *)(_CLKC_ADDR + 0x24))
  #define REFOCON_SET      (*(volatile unsigned *)(_CLKC_ADDR + 0x28))
  #define REFOCON_INV      (*(volatile unsigned *)(_CLKC_ADDR + 0x2C))
  #define REFOTRIM         (*(volatile unsigned *)(_CLKC_ADDR + 0x30))
  #define REFOTRIM_CLR     (*(volatile unsigned *)(_CLKC_ADDR + 0x34))
  #define REFOTRIM_SET     (*(volatile unsigned *)(_CLKC_ADDR + 0x38))
  #define REFOTRIM_INV     (*(volatile unsigned *)(_CLKC_ADDR + 0x3C))

    #define OSCCON_OSWEN                   0x00
    #define OSCCON_SOSCEN                  0x01
    #define OSCCON_UFRCEN                  0x02
    #define OSCCON_CF                      0x03
    #define OSCCON_SLPEN                   0x04
    #define OSCCON_SLOCK                   0x05
    #define OSCCON_ULOCK                   0x06
    #define OSCCON_CLKLOCK                 0x07
    #define OSCCON_NOSC                    0x08
    #define OSCCON_COSC                    0x0C
    #define OSCCON_PLLMULT                 0x10
    #define OSCCON_PBDIV                   0x13
    #define OSCCON_PBDIVRDY                0x15
    #define OSCCON_SOSCRDY                 0x16
    #define OSCCON_FRCDIV                  0x18
    #define OSCCON_PLLODIV                 0x1B
    #define OSCCON_NOSC                    0x08
    #define OSCCON_COSC                    0x0C
    #define OSCTUN_TUN                     0x00
    #define REFOCON_ROSEL                  0x00
    #define REFOCON_ACTIVE                 0x08
    #define REFOCON_DIVSWEN                0x09
    #define REFOCON_RSLP                   0x0B
    #define REFOCON_OE                     0x0C
    #define REFOCON_SIDL                   0x0D
    #define REFOCON_ON                     0x0F
    #define REFOCON_RODIV                  0x10
    #define REFOTRIM_ROTRIM                0x17

static inline unsigned __clkc_get_sysclk0(int cosc) {
    unsigned freq = 8000000;
    int div, pllmul;
    switch (cosc) {
    case 7:
	div = ( OSCCON >> OSCCON_FRCDIV ) & 0x7;
	if (div == 7) div++;
	freq >>= div;
	break;
    case 6:
	freq /= 16;
	break;
    case 5:
	freq = 32768;
	break;
    case 4:
	freq = 31250;
	break;
    case 3:
    case 1:
	pllmul = ( OSCCON >> OSCCON_PLLMULT ) & 0x7;
	if (pllmul == 7) pllmul += 2;
	pllmul += 15;
	freq = freq/2 * pllmul; // assume 4M (4M <= freq && freq <= 5M)
	div = ( OSCCON >> OSCCON_PLLODIV ) & 0x7;
	if (div == 7) div++;
	freq >>= div;
    }
    return freq;
}

static inline unsigned __clkc_get_sysclk() {
    int cosc = ( OSCCON >> OSCCON_COSC ) & 0x7;
    return __clkc_get_sysclk0(cosc);
}

static inline unsigned __clkc_get_pbclk0(int cosc) {
    int div = ( OSCCON >> OSCCON_PBDIV ) & 0x3;
    return __clkc_get_sysclk0(cosc) >> div;
}

static inline unsigned __clkc_get_pbclk() {
    int cosc = ( OSCCON >> OSCCON_COSC ) & 0x7;
    int div = ( OSCCON >> OSCCON_PBDIV ) & 0x3;
    return __clkc_get_sysclk0(cosc) >> div;
}
#endif /* _JZ_P32MX_CLKC_H_ */
